@@ -94,6 +94,29 @@ static int do_read(int fd, void *buf, size_t count)
    - we won't be in groupd cpg until after we've verified there's no
      local residual gfs/dlm state */
 
+static int ignore_gfs_nolock(char *sysfs_dir, char *table)
+{
+	char path[PATH_MAX];
+	int rv, fd;
+
+	if (!strstr(sysfs_dir, "gfs"))
+		return 0;
+
+	memset(path, 0, PATH_MAX);
+
+	snprintf(path, PATH_MAX, "%s/%s/lock_module/proto_name",
+		 sysfs_dir, table);
+
+	/* lock_nolock doesn't create the "lock_module" dir at all,
+	   so we'll fail to open this */
+
+	fd = open(path, O_RDONLY);
+	if (fd < 0)
+		return 1;
+
+	return 0;
+}
+
 static int kernel_instance_count(char *sysfs_dir)
 {
 	char path[PATH_MAX];
@@ -112,6 +135,9 @@ static int kernel_instance_count(char *sysfs_dir)
 		if (de->d_name[0] == '.')
 			continue;
 
+		if (ignore_gfs_nolock(sysfs_dir, de->d_name))
+			continue;
+
 		log_print("found uncontrolled kernel object %s in %s",
 			  de->d_name, sysfs_dir);
 		rv++;
@@ -126,8 +152,6 @@ int check_uncontrolled_groups(void)
 	char *argv[4];
 	int status, rv = 0;
 
-	/* FIXME: ignore gfs/gfs2 nolock fs's */
-
 	rv += kernel_instance_count("/sys/kernel/dlm");
 	rv += kernel_instance_count("/sys/fs/gfs");
 	rv += kernel_instance_count("/sys/fs/gfs2");
