@@ -1,25 +1,5 @@
 #!/bin/bash
 
-#
-#  Copyright Red Hat, Inc. 2004
-#  Copyright Mission Critical Linux, Inc. 2000
-#
-#  This program is free software; you can redistribute it and/or modify it
-#  under the terms of the GNU General Public License as published by the
-#  Free Software Foundation; either version 2, or (at your option) any
-#  later version.
-#
-#  This program is distributed in the hope that it will be useful, but
-#  WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; see the file COPYING.  If not, write to the
-#  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-#  MA 02139, USA.
-#
-
 #
 # IPv4/IPv6 address management using new /sbin/ifcfg instead of 
 # ifconfig utility.
@@ -110,6 +90,17 @@ meta_data()
 	    <content type="boolean"/>
 	</parameter>
 
+	<parameter name="sleeptime">
+	    <longdesc lang="en">
+		Amount of time to sleep after removing an IP address.
+		Value is specified in seconds. Default value is 10.
+	    </longdesc>
+	    <shortdesc lang="en">
+		Amount of time (seconds) to sleep.
+	    </shortdesc>
+	    <content type="string"/>
+	</parameter>
+
     </parameters>
 
     <actions>
@@ -542,13 +533,22 @@ network_link_up()
 	#
 	if [ "${intf_arg/bond/}" != "$intf_arg" ]; then
 		
+		#
+		# With Xen we must check for slaves of pbondX, not bondX
+		#
+		masterif=$intf_arg
+		line=$(ip link list | awk {'print $2'} | grep "p$intf_arg:")
+		if [ $? -eq 0 ] ; then
+			masterif="p$intf_arg"
+		fi
+		
 		#
 		# Bonded driver.  Check link of all slaves for this interface.
 		# If any link is up, the bonding driver is expected to route
 		# traffic through that link.  Thus, the entire bonded link
 		# is declared up.
 		#
-		slaves=$(findSlaves $intf_arg)
+		slaves=$(findSlaves $masterif)
 		if [ $? -ne 0 ]; then
 			ocf_log err "Error finding slaves of $intf_arg"
 			return 1
@@ -694,7 +694,7 @@ ipv4()
 			if [ "${addr/\/*/}" != "$ifaddr" ]; then
 			        continue
 			fi
-			addr=`/sbin/ip addr list | grep "$addr" | head -n 1 | awk '{print $2}'`
+			addr=`/sbin/ip addr list | grep "$addr/" | head -n 1 | awk '{print $2}'`
 			ocf_log info "Removing IPv4 address $addr from $dev"
 		fi
 		
@@ -907,7 +907,13 @@ stop)
 
 		# XXX Let nfsd/lockd clear their queues; we hope to have a
 		# way to enforce this in the future
-		sleep 10
+		if [ -z "$OCF_RESKEY_sleeptime" ]; then
+		    sleep 10
+		else
+		    if [ "$OCF_RESKEY_sleeptime" -gt "0" ]; then
+			sleep $OCF_RESKEY_sleeptime
+		    fi
+		fi
 	else
 		ocf_log debug "${OCF_RESKEY_address} is not configured"
 	fi
